'////////////////////////////////////////////////////////////////////////
' OSD Deploy Tool Branding Script
'////////////////////////////////////////////////////////////////////////
' Brand OSD variables to registry
'////////////////////////////////////////////////////////////////////////
' V2.00 6.11.2009 MICHS
'////////////////////////////////////////////////////////////////////////
' Include/Exclude
'////////////////////////////////////////////////////////////////////////
' 1. Include or exclude variables "starting with"
' 2. Use semicolon to separate multiple values
' 3. Exclude takes precedence over includes
'////////////////////////////////////////////////////////////////////////
Option Explicit

	'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
	' Constants
	'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
	Const	REG32			= "%windir%\System32\reg.exe"
	Const	REG64			= "%windir%\Sysnative\reg.exe"
	Const	REGBRANDPATH	= "HKLM\Software\WOW6432Node\Microsoft\MPSD\OSD"

	Const   tsAppVariableName      = "TsApplicationBaseVariable"
    Const   tsWindowsAppPackageAppVariableName = "TsWindowsAppPackageAppBaseVariable"
    Const   tsAppInstall           = "TsAppInstall"
    Const	includeMap		= "OSD;_SMSTSClientGUID;_SMSTSClientIdentity;USMT_;TSType;TSVersion;OldComputerName;PACKAGES;OSDBaseVariableName;DeploymentType"
	Const	excludeMap		= "OSDJoinPassword;_SMSTSReserved;OSDLocalAdminPassword"

	'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
	' Globals
	'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
	Dim REGBRAND
	Dim oWSH 				: SET oWSH = CreateObject("WScript.Shell")
	Dim oTSE				: SET oTSE = CreateObject("Microsoft.SMS.TSEnvironment")

	'[##############################################################################################################################]
	' MAIN
	'[##############################################################################################################################]

	'||||||||||||||||||||||||||||||||
	' Determine 32/64 Sysnative
	'||||||||||||||||||||||||||||||||
	Call LogArea("Environmental Setup")
	IF ( IsSysnative() = TRUE ) Then REGBRAND = REG64 Else REGBRAND = REG32

	'||||||||||||||||||||||||||||||||
	' Build Exclude/Include Arrays
	'||||||||||||||||||||||||||||||||
	Call LogArea("Mapping Inclusions and Exclusions")

    Dim applicationsPrefix : applicationsPrefix = oTSE(tsAppVariableName)
    Dim windowsAppPackageAppPrefix : windowsAppPackageAppPrefix = oTSE(tsWindowsAppPackageAppVariableName)
    Dim appInstall : appInstall = oTSE(tsAppInstall)

    'Brand Base Variable Values
    'And Variables that start with these Prefixes

    Dim incArray : incArray = Split( includeMap & ";" & tsAppVariableName & ";" & tsAppInstall & ";" & appInstall & ";" & tsWindowsAppPackageAppVariableName & ";" & windowsAppPackageAppPrefix  , ";" )
	Dim excArray : excArray = Split( excludeMap, ";" )

	'||||||||||||||||||||||||||||||||
	' Loop through TS Variables
	'||||||||||||||||||||||||||||||||
	Call LogArea("Branding Registry")
	Call BrandValue( "InstalledOn", Date )

	Dim tV
    For Each tV in oTSE.GetVariables()
		IF (MatchMaker( tV, incArray ) = TRUE) Then
			IF (MatchMaker( tV, excArray ) = FALSE ) Then
				Call BrandValue( tV, oTSE(tV) )
			End IF
		End IF
    Next

   'Brand Applications
    For Each tV in oTSE.GetVariables()
        If ( InStr(1, tV, applicationsPrefix, 1) = 1 ) Then
                    Call BrandValue( Replace(tV, applicationsPrefix, UCase(applicationsPrefix) & "0",1,-1, 1), oTSE(tV) )
		End IF
    Next

	WScript.Quit(0)

	'[##############################################################################################################################]
	' FUNCTIONS
	'[##############################################################################################################################]

	' ////////////////////////////////////////////////////
	' Brand a name and value to registry
	' ////////////////////////////////////////////////////
	Sub BrandValue( theName, theValue )

		Dim retVal : retVal = 0
		Dim runCmd : runCmd = REGBRAND & " ADD " & REGBRANDPATH & " /F /V " & theName & " /T REG_SZ /D """ & theValue & """"

		Wscript.Echo " Branding : [" & runCmd & "]"
		retVal = oWSH.Run( runCMD, 0, True )
		Wscript.Echo " Result   : [" & retVal & "]"

	End Sub

	' ////////////////////////////////////////////////////
	' Match "StartsWith" against an array of values
	' ////////////////////////////////////////////////////
	Function MatchMaker(theItem, theArray)
		Dim retVal : retVal = FALSE

		Dim anItem
		For Each anItem in theArray
			If ( Len(anItem)=0 ) Then Exit For
			' ||||||||||||||||||||||||||||||||
			'  - StartsWith is position 1
			'  - Case/Text Insensitive is 1
			' ||||||||||||||||||||||||||||||||
			If ( InStr(1, theItem, anItem, 1) = 1 ) Then
				retVal = TRUE
				Exit For
			End If
		Next

		MatchMaker = retVal

	End Function

	' ////////////////////////////////////////////////////
	' Detects if 32-bit environment on 64-bit OS
	' ////////////////////////////////////////////////////
	Function IsSysnative()

		Dim	PARCH1 : PARCH1 = UCASE( oWSH.ExpandEnvironmentStrings("%PROCESSOR_ARCHITECTURE%") )
		Dim	PARCH2 : PARCH2 = UCASE( oWSH.ExpandEnvironmentStrings("%PROCESSOR_ARCHITEW6432%") )

		wscript.echo "%PROCESSOR_ARCHITECTURE% = [" & PARCH1 & "]"
		wscript.echo "%PROCESSOR_ARCHITEW6432% = [" & PARCH2 & "]"

		IF ( (PARCH1 = "X86") AND (PARCH2 = "AMD64") ) Then IsSysnative=TRUE _
		ELSE IsSysnative = FALSE

		wscript.echo "32-BIT Environment on a 64-BIT OS: [" & IsSysnative & "]"

	End Function

	' ////////////////////////////////////////////////////
	' Log Area
	' ////////////////////////////////////////////////////
	Sub LogArea( theText )

		Wscript.Echo
		Wscript.Echo "---------------------------------------------------"
		Wscript.Echo " " & theText
		Wscript.Echo "---------------------------------------------------"
		Wscript.Echo

	End Sub
'' SIG '' Begin signature block
'' SIG '' MIIaSAYJKoZIhvcNAQcCoIIaOTCCGjUCAQExCzAJBgUr
'' SIG '' DgMCGgUAMGcGCisGAQQBgjcCAQSgWTBXMDIGCisGAQQB
'' SIG '' gjcCAR4wJAIBAQQQTvApFpkntU2P5azhDxfrqwIBAAIB
'' SIG '' AAIBAAIBAAIBADAhMAkGBSsOAwIaBQAEFM3KdrSqJuAj
'' SIG '' kwtlbQNllcJ16BjwoIIVLTCCBKAwggOIoAMCAQICCmEZ
'' SIG '' zJMAAQAAAGYwDQYJKoZIhvcNAQEFBQAweTELMAkGA1UE
'' SIG '' BhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNV
'' SIG '' BAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBD
'' SIG '' b3Jwb3JhdGlvbjEjMCEGA1UEAxMaTWljcm9zb2Z0IENv
'' SIG '' ZGUgU2lnbmluZyBQQ0EwHhcNMTExMDEwMjAzMjI1WhcN
'' SIG '' MTMwMTEwMjAzMjI1WjCBgzELMAkGA1UEBhMCVVMxEzAR
'' SIG '' BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
'' SIG '' bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
'' SIG '' bjENMAsGA1UECxMETU9QUjEeMBwGA1UEAxMVTWljcm9z
'' SIG '' b2Z0IENvcnBvcmF0aW9uMIIBIjANBgkqhkiG9w0BAQEF
'' SIG '' AAOCAQ8AMIIBCgKCAQEA7lu+fREk44YG4Gb/SLUXvQLk
'' SIG '' tAwy8HI+fS6H106hsadDL/dlnjHhMjFFrtfBJIQh1y61
'' SIG '' hH76NdNTHNe2UR5Pzma567cMAv0pXK2oh/bKIrTVvwh1
'' SIG '' 9Ypwj2PX74oe6Y9DJGRa04d9kG07rHbNVzZ96LwQVqyY
'' SIG '' 8IldLmTGryYJXh5jFfE9vxaPmYgCwzC3wQtgHw9yzNa3
'' SIG '' qDUShpuhCwrmk1uO+lScwfMZX0KNEp8dP5C3JxODGTKC
'' SIG '' HfPZh9QhsjyitgdP1ySq7o31s9n6+TlPp+nyr1lS9NxB
'' SIG '' my8RcGPd6t6q8W0hBBBTM7uyT8XhU7JBZUduN/a86ZsW
'' SIG '' QZFrLlswwwIDAQABo4IBHTCCARkwEwYDVR0lBAwwCgYI
'' SIG '' KwYBBQUHAwMwHQYDVR0OBBYEFBtSDvMRKrfAicMRgT3U
'' SIG '' lli5o1NuMA4GA1UdDwEB/wQEAwIHgDAfBgNVHSMEGDAW
'' SIG '' gBTLEejK0rQWWAHJNy4zFha5TJoKHzBWBgNVHR8ETzBN
'' SIG '' MEugSaBHhkVodHRwOi8vY3JsLm1pY3Jvc29mdC5jb20v
'' SIG '' cGtpL2NybC9wcm9kdWN0cy9NaWNDb2RTaWdQQ0FfMDgt
'' SIG '' MzEtMjAxMC5jcmwwWgYIKwYBBQUHAQEETjBMMEoGCCsG
'' SIG '' AQUFBzAChj5odHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20v
'' SIG '' cGtpL2NlcnRzL01pY0NvZFNpZ1BDQV8wOC0zMS0yMDEw
'' SIG '' LmNydDANBgkqhkiG9w0BAQUFAAOCAQEApVs2bK1Om2kS
'' SIG '' 42+KAptpd8NsZHIoiNk9RW0sGHvUKC8T4llqG8ILNLeK
'' SIG '' /eq5lOwHMeZq9HUE06faXjoGnhD9qQ29nFFDb/9nlJzh
'' SIG '' z3zwJLA1zINd7trAbzZJwFoKV/Zz4Z7z4whMOz4vzNLN
'' SIG '' 7k8icPcEHwKmS5u4j1yIDjaUbDMHuKmtUaDQwtyOIhK9
'' SIG '' w9+C11ah993wpSBXEBCd7qyGdGxxm8Hw8sJwXqfbbU03
'' SIG '' WJlNeUDQNF1aJa5n6xtORgawjCkfoxCPpTOfI9X4tUZ9
'' SIG '' 4O5jmJBLPgWoL7AYs1mkr0FTjggFEC0qyToGTBwuqTFR
'' SIG '' VmSsmsysl5gpipeQh+qdtjCCBLowggOioAMCAQICCmEC
'' SIG '' kkoAAAAAACAwDQYJKoZIhvcNAQEFBQAwdzELMAkGA1UE
'' SIG '' BhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNV
'' SIG '' BAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBD
'' SIG '' b3Jwb3JhdGlvbjEhMB8GA1UEAxMYTWljcm9zb2Z0IFRp
'' SIG '' bWUtU3RhbXAgUENBMB4XDTEyMDEwOTIyMjU1OVoXDTEz
'' SIG '' MDQwOTIyMjU1OVowgbMxCzAJBgNVBAYTAlVTMRMwEQYD
'' SIG '' VQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25k
'' SIG '' MR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24x
'' SIG '' DTALBgNVBAsTBE1PUFIxJzAlBgNVBAsTHm5DaXBoZXIg
'' SIG '' RFNFIEVTTjpCOEVDLTMwQTQtNzE0NDElMCMGA1UEAxMc
'' SIG '' TWljcm9zb2Z0IFRpbWUtU3RhbXAgU2VydmljZTCCASIw
'' SIG '' DQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAM1jw/ei
'' SIG '' tUfZ+TmUU6xrj6Z5OCH00W49FTgWwXMsmY/74Dxb4aJM
'' SIG '' i7Kri7TySse5k1DRJvWHU7B6dfNHDxcrZyxk62DnSozg
'' SIG '' i17EVmk3OioEXRcByL+pt9PJq6ORqIHjPy232OTEeAB5
'' SIG '' Oc/9x2TiIxJ4ngx2J0mPmqwOdOMGVVVJyO2hfHBFYX6y
'' SIG '' cRYe4cFBudLSMulSJPM2UATX3W88SdUL1HZA/GVlE36V
'' SIG '' UTrV/7iap1drSxXlN1gf3AANxa7q34FH+fBSrubPWqzg
'' SIG '' FEqmcZSA+v2wIzBg6YNgrA4kHv8R8uelVWKV7p9/ninW
'' SIG '' zUsKdoPwQwTfBkkg8lNaRLBRejkCAwEAAaOCAQkwggEF
'' SIG '' MB0GA1UdDgQWBBTNGaxhTZRnK/avlHVZ2/BYAIOhOjAf
'' SIG '' BgNVHSMEGDAWgBQjNPjZUkZwCu1A+3b7syuwwzWzDzBU
'' SIG '' BgNVHR8ETTBLMEmgR6BFhkNodHRwOi8vY3JsLm1pY3Jv
'' SIG '' c29mdC5jb20vcGtpL2NybC9wcm9kdWN0cy9NaWNyb3Nv
'' SIG '' ZnRUaW1lU3RhbXBQQ0EuY3JsMFgGCCsGAQUFBwEBBEww
'' SIG '' SjBIBggrBgEFBQcwAoY8aHR0cDovL3d3dy5taWNyb3Nv
'' SIG '' ZnQuY29tL3BraS9jZXJ0cy9NaWNyb3NvZnRUaW1lU3Rh
'' SIG '' bXBQQ0EuY3J0MBMGA1UdJQQMMAoGCCsGAQUFBwMIMA0G
'' SIG '' CSqGSIb3DQEBBQUAA4IBAQBRHNbfNh3cgLwCp8aZ3xbI
'' SIG '' kAZpFZoyufNkENKK82IpG3mPymCps13E5BYtNYxEm/H0
'' SIG '' XGGkQa6ai7pQ0Wp5arNijJ1NUVALqY7Uv6IQwEfVTnVS
'' SIG '' iR4/lmqPLkAUBnLuP3BZkl2F7YOZ+oKEnuQDASETqyfW
'' SIG '' zHFJ5dod/288CU7VjWboDMl/7jEUAjdfe2nsiT5FfyVE
'' SIG '' 5x8a1sUaw0rk4fGEmOdP+amYpxhG7IRs7KkDCv18elId
'' SIG '' nGukqA+YkqSSeFwreON9ssfZtnB931tzU7+q1GZQS/DJ
'' SIG '' O5WF5cFKZZ0lWFC7IFSReTobB1xqVyivMcef58Md7kf9
'' SIG '' J9d/z3TcZcU/MIIFvDCCA6SgAwIBAgIKYTMmGgAAAAAA
'' SIG '' MTANBgkqhkiG9w0BAQUFADBfMRMwEQYKCZImiZPyLGQB
'' SIG '' GRYDY29tMRkwFwYKCZImiZPyLGQBGRYJbWljcm9zb2Z0
'' SIG '' MS0wKwYDVQQDEyRNaWNyb3NvZnQgUm9vdCBDZXJ0aWZp
'' SIG '' Y2F0ZSBBdXRob3JpdHkwHhcNMTAwODMxMjIxOTMyWhcN
'' SIG '' MjAwODMxMjIyOTMyWjB5MQswCQYDVQQGEwJVUzETMBEG
'' SIG '' A1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9u
'' SIG '' ZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
'' SIG '' MSMwIQYDVQQDExpNaWNyb3NvZnQgQ29kZSBTaWduaW5n
'' SIG '' IFBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoC
'' SIG '' ggEBALJyWVwZMGS/HZpgICBCmXZTbD4b1m/My/Hqa/6X
'' SIG '' FhDg3zp0gxq3L6Ay7P/ewkJOI9VyANs1VwqJyq4gSfTw
'' SIG '' aKxNS42lvXlLcZtHB9r9Jd+ddYjPqnNEf9eB2/O98jak
'' SIG '' yVxF3K+tPeAoaJcap6Vyc1bxF5Tk/TWUcqDWdl8ed0WD
'' SIG '' hTgW0HNbBbpnUo2lsmkv2hkL/pJ0KeJ2L1TdFDBZ+NKN
'' SIG '' Yv3LyV9GMVC5JxPkQDDPcikQKCLHN049oDI9kM2hOAaF
'' SIG '' XE5WgigqBTK3S9dPY+fSLWLxRT3nrAgA9kahntFbjCZT
'' SIG '' 6HqqSvJGzzc8OJ60d1ylF56NyxGPVjzBrAlfA9MCAwEA
'' SIG '' AaOCAV4wggFaMA8GA1UdEwEB/wQFMAMBAf8wHQYDVR0O
'' SIG '' BBYEFMsR6MrStBZYAck3LjMWFrlMmgofMAsGA1UdDwQE
'' SIG '' AwIBhjASBgkrBgEEAYI3FQEEBQIDAQABMCMGCSsGAQQB
'' SIG '' gjcVAgQWBBT90TFO0yaKleGYYDuoMW+mPLzYLTAZBgkr
'' SIG '' BgEEAYI3FAIEDB4KAFMAdQBiAEMAQTAfBgNVHSMEGDAW
'' SIG '' gBQOrIJgQFYnl+UlE/wq4QpTlVnkpDBQBgNVHR8ESTBH
'' SIG '' MEWgQ6BBhj9odHRwOi8vY3JsLm1pY3Jvc29mdC5jb20v
'' SIG '' cGtpL2NybC9wcm9kdWN0cy9taWNyb3NvZnRyb290Y2Vy
'' SIG '' dC5jcmwwVAYIKwYBBQUHAQEESDBGMEQGCCsGAQUFBzAC
'' SIG '' hjhodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtpL2Nl
'' SIG '' cnRzL01pY3Jvc29mdFJvb3RDZXJ0LmNydDANBgkqhkiG
'' SIG '' 9w0BAQUFAAOCAgEAWTk+fyZGr+tvQLEytWrrDi9uqEn3
'' SIG '' 61917Uw7LddDrQv+y+ktMaMjzHxQmIAhXaw9L0y6oqhW
'' SIG '' nONwu7i0+Hm1SXL3PupBf8rhDBdpy6WcIC36C1DEVs0t
'' SIG '' 40rSvHDnqA2iA6VW4LiKS1fylUKc8fPv7uOGHzQ8uFaa
'' SIG '' 8FMjhSqkghyT4pQHHfLiTviMocroE6WRTsgb0o9ylSpx
'' SIG '' bZsa+BzwU9ZnzCL/XB3Nooy9J7J5Y1ZEolHN+emjWFbd
'' SIG '' mwJFRC9f9Nqu1IIybvyklRPk62nnqaIsvsgrEA5ljpnb
'' SIG '' 9aL6EiYJZTiU8XofSrvR4Vbo0HiWGFzJNRZf3ZMdSY4t
'' SIG '' vq00RBzuEBUaAF3dNVshzpjHCe6FDoxPbQ4TTj18KUic
'' SIG '' ctHzbMrB7HCjV5JXfZSNoBtIA1r3z6NnCnSlNu0tLxfI
'' SIG '' 5nI3EvRvsTxngvlSso0zFmUeDordEN5k9G/ORtTTF+l5
'' SIG '' xAS00/ss3x+KnqwK+xMnQK3k+eGpf0a7B2BHZWBATrBC
'' SIG '' 7E7ts3Z52Ao0CW0cgDEf4g5U3eWh++VHEK1kmP9QFi58
'' SIG '' vwUheuKVQSdpw5OPlcmN2Jshrg1cnPCiroZogwxqLbt2
'' SIG '' awAdlq3yFnv2FoMkuYjPaqhHMS+a3ONxPdcAfmJH0c6I
'' SIG '' ybgY+g5yjcGjPa8CQGr/aZuW4hCoELQ3UAjWwz0wggYH
'' SIG '' MIID76ADAgECAgphFmg0AAAAAAAcMA0GCSqGSIb3DQEB
'' SIG '' BQUAMF8xEzARBgoJkiaJk/IsZAEZFgNjb20xGTAXBgoJ
'' SIG '' kiaJk/IsZAEZFgltaWNyb3NvZnQxLTArBgNVBAMTJE1p
'' SIG '' Y3Jvc29mdCBSb290IENlcnRpZmljYXRlIEF1dGhvcml0
'' SIG '' eTAeFw0wNzA0MDMxMjUzMDlaFw0yMTA0MDMxMzAzMDla
'' SIG '' MHcxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5n
'' SIG '' dG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVN
'' SIG '' aWNyb3NvZnQgQ29ycG9yYXRpb24xITAfBgNVBAMTGE1p
'' SIG '' Y3Jvc29mdCBUaW1lLVN0YW1wIFBDQTCCASIwDQYJKoZI
'' SIG '' hvcNAQEBBQADggEPADCCAQoCggEBAJ+hbLHf20iSKnxr
'' SIG '' LhnhveLjxZlRI1Ctzt0YTiQP7tGn0UytdDAgEesH1VSV
'' SIG '' FUmUG0KSrphcMCbaAGvoe73siQcP9w4EmPCJzB/LMySH
'' SIG '' nfL0Zxws/HvniB3q506jocEjU8qN+kXPCdBer9CwQgSi
'' SIG '' +aZsk2fXKNxGU7CG0OUoRi4nrIZPVVIM5AMs+2qQkDBu
'' SIG '' h/NZMJ36ftaXs+ghl3740hPzCLdTbVK0RZCfSABKR2YR
'' SIG '' JylmqJfk0waBSqL5hKcRRxQJgp+E7VV4/gGaHVAIhQAQ
'' SIG '' MEbtt94jRrvELVSfrx54QTF3zJvfO4OToWECtR0Nsfz3
'' SIG '' m7IBziJLVP/5BcPCIAsCAwEAAaOCAaswggGnMA8GA1Ud
'' SIG '' EwEB/wQFMAMBAf8wHQYDVR0OBBYEFCM0+NlSRnAK7UD7
'' SIG '' dvuzK7DDNbMPMAsGA1UdDwQEAwIBhjAQBgkrBgEEAYI3
'' SIG '' FQEEAwIBADCBmAYDVR0jBIGQMIGNgBQOrIJgQFYnl+Ul
'' SIG '' E/wq4QpTlVnkpKFjpGEwXzETMBEGCgmSJomT8ixkARkW
'' SIG '' A2NvbTEZMBcGCgmSJomT8ixkARkWCW1pY3Jvc29mdDEt
'' SIG '' MCsGA1UEAxMkTWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNh
'' SIG '' dGUgQXV0aG9yaXR5ghB5rRahSqClrUxzWPQHEy5lMFAG
'' SIG '' A1UdHwRJMEcwRaBDoEGGP2h0dHA6Ly9jcmwubWljcm9z
'' SIG '' b2Z0LmNvbS9wa2kvY3JsL3Byb2R1Y3RzL21pY3Jvc29m
'' SIG '' dHJvb3RjZXJ0LmNybDBUBggrBgEFBQcBAQRIMEYwRAYI
'' SIG '' KwYBBQUHMAKGOGh0dHA6Ly93d3cubWljcm9zb2Z0LmNv
'' SIG '' bS9wa2kvY2VydHMvTWljcm9zb2Z0Um9vdENlcnQuY3J0
'' SIG '' MBMGA1UdJQQMMAoGCCsGAQUFBwMIMA0GCSqGSIb3DQEB
'' SIG '' BQUAA4ICAQAQl4rDXANENt3ptK132855UU0BsS50cVtt
'' SIG '' DBOrzr57j7gu1BKijG1iuFcCy04gE1CZ3XpA4le7r1ia
'' SIG '' HOEdAYasu3jyi9DsOwHu4r6PCgXIjUji8FMV3U+rkuTn
'' SIG '' jWrVgMHmlPIGL4UD6ZEqJCJw+/b85HiZLg33B+JwvBhO
'' SIG '' nY5rCnKVuKE5nGctxVEO6mJcPxaYiyA/4gcaMvnMMUp2
'' SIG '' MT0rcgvI6nA9/4UKE9/CCmGO8Ne4F+tOi3/FNSteo7/r
'' SIG '' vH0LQnvUU3Ih7jDKu3hlXFsBFwoUDtLaFJj1PLlmWLMt
'' SIG '' L+f5hYbMUVbonXCUbKw5TNT2eb+qGHpiKe+imyk0Bnca
'' SIG '' Ysk9Hm0fgvALxyy7z0Oz5fnsfbXjpKh0NbhOxXEjEiZ2
'' SIG '' CzxSjHFaRkMUvLOzsE1nyJ9C/4B5IYCeFTBm6EISXhrI
'' SIG '' niIh0EPpK+m79EjMLNTYMoBMJipIJF9a6lbvpt6Znco6
'' SIG '' b72BJ3QGEe52Ib+bgsEnVLaxaj2JoXZhtG6hE6a/qkfw
'' SIG '' Em/9ijJssv7fUciMI8lmvZ0dhxJkAj0tr1mPuOQh5bWw
'' SIG '' ymO0eFQF1EEuUKyUsKV4q7OglnUa2ZKHE3UiLzKoCG6g
'' SIG '' W4wlv6DvhMoh1useT8ma7kng9wFlb4kLfchpyOZu6qeX
'' SIG '' zjEp/w7FW1zYTRuh2Povnj8uVRZryROj/TGCBIcwggSD
'' SIG '' AgEBMIGHMHkxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpX
'' SIG '' YXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYD
'' SIG '' VQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xIzAhBgNV
'' SIG '' BAMTGk1pY3Jvc29mdCBDb2RlIFNpZ25pbmcgUENBAgph
'' SIG '' GcyTAAEAAABmMAkGBSsOAwIaBQCggbIwGQYJKoZIhvcN
'' SIG '' AQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEO
'' SIG '' MAwGCisGAQQBgjcCARUwIwYJKoZIhvcNAQkEMRYEFILO
'' SIG '' YI/P8JuuQ1fniAdlqGbsjRXgMFIGCisGAQQBgjcCAQwx
'' SIG '' RDBCoCSAIgBNAEQAVAAgAFUARABJAHYAMwAgAFQAbwBv
'' SIG '' AGwAawBpAHShGoAYaHR0cDovL3d3dy5taWNyb3NvZnQu
'' SIG '' Y29tMA0GCSqGSIb3DQEBAQUABIIBADA4iV2S+1VNsgsr
'' SIG '' wD0mb17rzxNgLSXpg8BFfqwz2mzre3gqBjKRqj+Id+kj
'' SIG '' CWEZ/DoVh7+lSu+/zZ0dSBatAA2WRs0g1pVLkwCllc/B
'' SIG '' 00745V9wRtd4PIXGehEM/L5nHmPLMYqFT/hE+SiapjvP
'' SIG '' MhjXC9pLy4uZ9JQTBOrXUE9hFXl2yG/DbgZRn+ct05C+
'' SIG '' Vd90GWb1/vuflrv8K0jw447TxzcAlcYqBgPwuWFGKa4B
'' SIG '' rqrYWQRbgOO7JQWxyJ50M7ZzVZ7s8B9h3WcFYnyfjQER
'' SIG '' 9vBVFbLisxoo2qshrumJUkRx7X5aY0AGVa8luip6Xgm8
'' SIG '' zOGM23nhaeRj4dYIHj2hggIfMIICGwYJKoZIhvcNAQkG
'' SIG '' MYICDDCCAggCAQEwgYUwdzELMAkGA1UEBhMCVVMxEzAR
'' SIG '' BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
'' SIG '' bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
'' SIG '' bjEhMB8GA1UEAxMYTWljcm9zb2Z0IFRpbWUtU3RhbXAg
'' SIG '' UENBAgphApJKAAAAAAAgMAkGBSsOAwIaBQCgXTAYBgkq
'' SIG '' hkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJ
'' SIG '' BTEPFw0xMjA4MDIyMzU2MzlaMCMGCSqGSIb3DQEJBDEW
'' SIG '' BBQNMDCQ0pCpWT9Mqy9qgU/pD4anUjANBgkqhkiG9w0B
'' SIG '' AQUFAASCAQBvI4/NAMLw6xzoM12yxpgBJJ9RzsOBqzTL
'' SIG '' ige2vY19D8l3gBxlfblgOwzlHddqnmHvEFtVFKASFirs
'' SIG '' MOWa8hyo2ORt8V+xXBtluMne8khce0RD7yRgGf2qatyo
'' SIG '' 4qTEGS6QYHQUA/XMZQBrnqwEuhpWm1laA6htaie2EAJt
'' SIG '' Umin8DxQQ3+JUSGlWtsNlOddnLqM/6VTi2U90gtXztmO
'' SIG '' tghq/RdKmQJ8lJFc3SAvoqRIT+QqVMdon7smS1MJTb08
'' SIG '' M+6SHrtGr+wOXJxRgTwvyWFXQQ+pFOdUqUIrI0g3hR+T
'' SIG '' KnD3q9faPXdODtl2/Fehg4KmYPWepS1ub3g9E/9kwYn8
'' SIG '' End signature block
