'///////////////////////////////////////////////////////
' OSD_BaseVariables
'///////////////////////////////////////////////////////
'08.19.09 | v1.0.0.0 | MICHS & BENSHY
'///////////////////////////////////////////////////////
Option Explicit

	' ############################################################ CONST

	Const TsXmlVarName    = "_SMSTSTaskSequence"
	Const TsBaseVarName   = "BaseVariableName"
	Const TsSaveVarName   = "OSDBaseVariableName"
    Const TsAppsVarName = "TsApplicationBaseVariable"
	Const OSDBrandingArea = "HKLM\SOFTWARE\Microsoft\MPSD\OSD"

	' ############################################################ GLOBAL
	
	Dim TsBaseValue
    Dim TsAppsBaseValue
	
	Dim oTSE
	Dim oXML
	Dim oWSH
	Dim oWMI

	' ############################################################ MAIN BEGIN
	
	PrintTitle("Initializing Objects")
	If (SetObjects = false) Then QuitScript( 100 )
	
	PrintTitle("Extracting TS Base Variable")
	If (GetBaseVariableName = false) Then QuitScript( 200 )
	
	PrintTitle("Extracting Package Names")
	If (GetPackageNames = false) Then QuitScript( 300 )
	
	PrintTitle("Script Completed Successfully!")
	QuitScript ( 0 )
	
	' ############################################################ MAIN END



	
	' /////////////////////////////////////////////////////////
	' Initialize/Set Objects
	' /////////////////////////////////////////////////////////
	Function SetObjects
		SetObjects = true

		On Error Resume Next
		Err.Number = 0
		
		SET oTSE = CreateObject("Microsoft.SMS.TSEnvironment") 
		SET oXML = CreateObject("Microsoft.XMLDOM")
		SET oWSH = CreateObject("WScript.Shell")
		SET oWMI = GetObject( "winmgmts:{impersonationLevel=impersonate}!\\.\root\ccm\Policy\Machine" )
	
		if (Err.Number <> 0) Then 
			SetObjects = false
			wscript.echo " --| Error: [" & Err.Number & "]"
			wscript.echo " --| Description: [" & Err.Description & "]"
		End If
	
		On Error Goto 0
	
	End Function	
	
	
	' /////////////////////////////////////////////////////////
	' Return Base Variable Name
	' /////////////////////////////////////////////////////////
	Function GetBaseVariableName
		GetBaseVariableName = true
		
		Dim xmlString
        Dim nodeList
	
		On Error Resume Next
		Err.Number = 0
	
		Wscript.Echo "Reading OSD variable: [" & TsXmlVarName & "]"
		xmlString = oTSE(TsXmlVarName)
		If (Len(xmlString)=0) Then
			Wscript.Echo "Failed to read OSD variable, or value is empty."
			GetBaseVariableName = false
			Exit Function
		End If
		
		wscript.Echo "Loading XML from variable string content..."
		if (oXML.LoadXml( xmlString ) = false) Then
			Wscript.Echo "Failed to load XML from string (OSD variable)."
			GetBaseVariableName = false
			Exit Function
		End If

		wscript.Echo "Using XPATH to query for [" & TsBaseVarName & "]"
		TsBaseValue = ""
		'TsBaseValue = oXML.selectSingleNode( "//variable[@name='" & TsBaseVarName & "']" ).Text
        Set nodeList = oXML.selectNodes( "//variable[@name='" & TsBaseVarName & "']" )
        TsBaseValue = nodeList.item(0).text
        TsAppsBaseValue = nodeList.item(1).text


		If (Len(TsBaseValue)=0) Then
			Wscript.Echo "Failed to read XPATH query value, or value is empty."
			GetBaseVariableName = false
			Exit Function
		End If
		
		wscript.echo "Extracted XPATH variable name: [" & TsBaseValue & "]"
		wscript.Echo "Setting new variable: [" & TsSaveVarName & "] = [" & TsBaseValue & "]"
		oTSE( TsSaveVarName ) = TsBaseValue
        oTSE( TsAppsVarName ) = TsAppsBaseValue

		If (Err.Number <> 0) Then 
			GetBaseVariableName = false
			wscript.echo " --| Error: [" & Err.Number & "]"
			wscript.echo " --| Description: [" & Err.Description & "]"
		End If
	
		On Error Goto 0
		
	End Function	
	

	' /////////////////////////////////////////////////////////
	' Get Package Names
	' /////////////////////////////////////////////////////////
	Function GetPackageNames
		GetPackageNames = true
		
		Dim numItem
		Dim numItemString
		Dim itemFound
		
		numItem       = 0
		numItemString = ""
		itemFound     = true
		
			Do 
				Dim nameBuilder
				Dim nameValue
			
				numItem = numItem + 1
				
				If (numItem<10) Then 
					numItemString = "00" & numItem
				ElseIf (numItem<100) Then 
					numItemString = "0" & numItem
				Else
					numItemString = numItem
				End If

				nameBuilder = TsBaseValue & numItemString
				wscript.echo "Checking for registry entry name: [" & nameBuilder & "]"

				nameValue = oTSE( nameBuilder )
				wscript.echo " --| Value: [" & nameValue & "]"

				If ( len(nameValue) = 0 ) Then 
					wscript.echo " --| Item not found."
					itemFound = false
				Else

					Dim wmiValue
					
					wmiValue = GetPackageName( nameValue )
					nameBuilder = nameBuilder & "Name"
					oTSE( nameBuilder ) = wmiValue
					
					wscript.echo " --| Set OSD variable [" & nameBuilder & "] equal to [" & wmiValue & "]"
					
				End If
				
				
			Loop Until (itemFound = false)
	
	End Function
	
	
	' /////////////////////////////////////////////////////////
	' Get Package Name from WMI
	' /////////////////////////////////////////////////////////
	Function GetPackageName( thePackageProgram )
		
		Dim PackageID
		Dim PackageName
		Dim splitArray

		splitArray = Split( thePackageProgram, ":" )
		PackageID  =( splitArray(0) )

		Dim WMICollection
		Dim WMIItem
		
		On Error Resume Next
		Err.Number = 0		

		wscript.echo " --| Running WMI Query..."
		SET WMICollection = oWMI.ExecQuery( "Select * from CCM_SoftwareDistribution where PKG_PackageID='" & PackageID & "'" )

		GetPackageName = ""
		For Each WMIItem in WMICollection
			GetPackageName = WMIItem.PKG_Name
			wscript.echo " --| Found: [" & GetPackageName & "]"
		Next

		If (Err.Number <> 0) Then 
			GetPackageName = ""
			wscript.echo " --| Error: [" & Err.Number & "]"
			wscript.echo " --| Description: [" & Err.Description & "]"
		End If
		
		On Error Goto 0
		
	End Function


	' /////////////////////////////////////////////////////////
	' Print Title
	' /////////////////////////////////////////////////////////
	Sub PrintTitle( theTitle )
	
		wscript.echo "--------------------------------"
		wscript.echo theTitle
		wscript.echo "--------------------------------"
		wscript.echo ""
		
	End Sub


	' /////////////////////////////////////////////////////////
	' Quit Script
	' /////////////////////////////////////////////////////////	
	Sub QuitScript( theExitCode )
	
		wscript.echo "--------------------------------"
		wscript.echo " Exiting with [" & theExitCode & "]"
		wscript.echo "--------------------------------"	

		wscript.Quit( theExitCode )
	
	End Sub
'' SIG '' Begin signature block
'' SIG '' MIIaOAYJKoZIhvcNAQcCoIIaKTCCGiUCAQExCzAJBgUr
'' SIG '' DgMCGgUAMGcGCisGAQQBgjcCAQSgWTBXMDIGCisGAQQB
'' SIG '' gjcCAR4wJAIBAQQQTvApFpkntU2P5azhDxfrqwIBAAIB
'' SIG '' AAIBAAIBAAIBADAhMAkGBSsOAwIaBQAEFM1lvtpCJvwC
'' SIG '' dzqKW/7u30lyLsfFoIIVLTCCBKAwggOIoAMCAQICCmEZ
'' SIG '' zJMAAQAAAGYwDQYJKoZIhvcNAQEFBQAweTELMAkGA1UE
'' SIG '' BhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNV
'' SIG '' BAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBD
'' SIG '' b3Jwb3JhdGlvbjEjMCEGA1UEAxMaTWljcm9zb2Z0IENv
'' SIG '' ZGUgU2lnbmluZyBQQ0EwHhcNMTExMDEwMjAzMjI1WhcN
'' SIG '' MTMwMTEwMjAzMjI1WjCBgzELMAkGA1UEBhMCVVMxEzAR
'' SIG '' BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
'' SIG '' bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
'' SIG '' bjENMAsGA1UECxMETU9QUjEeMBwGA1UEAxMVTWljcm9z
'' SIG '' b2Z0IENvcnBvcmF0aW9uMIIBIjANBgkqhkiG9w0BAQEF
'' SIG '' AAOCAQ8AMIIBCgKCAQEA7lu+fREk44YG4Gb/SLUXvQLk
'' SIG '' tAwy8HI+fS6H106hsadDL/dlnjHhMjFFrtfBJIQh1y61
'' SIG '' hH76NdNTHNe2UR5Pzma567cMAv0pXK2oh/bKIrTVvwh1
'' SIG '' 9Ypwj2PX74oe6Y9DJGRa04d9kG07rHbNVzZ96LwQVqyY
'' SIG '' 8IldLmTGryYJXh5jFfE9vxaPmYgCwzC3wQtgHw9yzNa3
'' SIG '' qDUShpuhCwrmk1uO+lScwfMZX0KNEp8dP5C3JxODGTKC
'' SIG '' HfPZh9QhsjyitgdP1ySq7o31s9n6+TlPp+nyr1lS9NxB
'' SIG '' my8RcGPd6t6q8W0hBBBTM7uyT8XhU7JBZUduN/a86ZsW
'' SIG '' QZFrLlswwwIDAQABo4IBHTCCARkwEwYDVR0lBAwwCgYI
'' SIG '' KwYBBQUHAwMwHQYDVR0OBBYEFBtSDvMRKrfAicMRgT3U
'' SIG '' lli5o1NuMA4GA1UdDwEB/wQEAwIHgDAfBgNVHSMEGDAW
'' SIG '' gBTLEejK0rQWWAHJNy4zFha5TJoKHzBWBgNVHR8ETzBN
'' SIG '' MEugSaBHhkVodHRwOi8vY3JsLm1pY3Jvc29mdC5jb20v
'' SIG '' cGtpL2NybC9wcm9kdWN0cy9NaWNDb2RTaWdQQ0FfMDgt
'' SIG '' MzEtMjAxMC5jcmwwWgYIKwYBBQUHAQEETjBMMEoGCCsG
'' SIG '' AQUFBzAChj5odHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20v
'' SIG '' cGtpL2NlcnRzL01pY0NvZFNpZ1BDQV8wOC0zMS0yMDEw
'' SIG '' LmNydDANBgkqhkiG9w0BAQUFAAOCAQEApVs2bK1Om2kS
'' SIG '' 42+KAptpd8NsZHIoiNk9RW0sGHvUKC8T4llqG8ILNLeK
'' SIG '' /eq5lOwHMeZq9HUE06faXjoGnhD9qQ29nFFDb/9nlJzh
'' SIG '' z3zwJLA1zINd7trAbzZJwFoKV/Zz4Z7z4whMOz4vzNLN
'' SIG '' 7k8icPcEHwKmS5u4j1yIDjaUbDMHuKmtUaDQwtyOIhK9
'' SIG '' w9+C11ah993wpSBXEBCd7qyGdGxxm8Hw8sJwXqfbbU03
'' SIG '' WJlNeUDQNF1aJa5n6xtORgawjCkfoxCPpTOfI9X4tUZ9
'' SIG '' 4O5jmJBLPgWoL7AYs1mkr0FTjggFEC0qyToGTBwuqTFR
'' SIG '' VmSsmsysl5gpipeQh+qdtjCCBLowggOioAMCAQICCmEF
'' SIG '' EzYAAAAAABowDQYJKoZIhvcNAQEFBQAwdzELMAkGA1UE
'' SIG '' BhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNV
'' SIG '' BAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBD
'' SIG '' b3Jwb3JhdGlvbjEhMB8GA1UEAxMYTWljcm9zb2Z0IFRp
'' SIG '' bWUtU3RhbXAgUENBMB4XDTExMDcyNTIwNDIxN1oXDTEy
'' SIG '' MTAyNTIwNDIxN1owgbMxCzAJBgNVBAYTAlVTMRMwEQYD
'' SIG '' VQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25k
'' SIG '' MR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24x
'' SIG '' DTALBgNVBAsTBE1PUFIxJzAlBgNVBAsTHm5DaXBoZXIg
'' SIG '' RFNFIEVTTjoxNTlDLUEzRjctMjU3MDElMCMGA1UEAxMc
'' SIG '' TWljcm9zb2Z0IFRpbWUtU3RhbXAgU2VydmljZTCCASIw
'' SIG '' DQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAJw0mBnJ
'' SIG '' CSljmQIWdaiIV32hu6lBEvxkkaXWBXi/d9zs5q64UE7A
'' SIG '' 4xq97bf9+CCKcTmqcpJyn4oJ5RPvkUHtYSVrUa3uqEO1
'' SIG '' YUIsnfIdsdL8t/V7o3N2E7Mro9uUYYBVoQ9t3djsFv+F
'' SIG '' f5aeiH8ALo56JLponY/GyvSQeXrhm+8GXX74LsNqFZw8
'' SIG '' FC/n1ZTIIhtRy6lVhiG3WvNBEjmf8FWpTqolK2P7kXI8
'' SIG '' D3zAlnLcwaPBCMrexcm1wApfpZwLqnIKXQpAfS6Y0Kuy
'' SIG '' iI+GgOh90b5Va+BYLMg2P/nmEcPwQwWFeAMX5SynKXzT
'' SIG '' 4pUXAGzb3K08GToC4H1i1M72HT0CAwEAAaOCAQkwggEF
'' SIG '' MB0GA1UdDgQWBBT2g4sghxHug8vb3oWi0miGN2F0kDAf
'' SIG '' BgNVHSMEGDAWgBQjNPjZUkZwCu1A+3b7syuwwzWzDzBU
'' SIG '' BgNVHR8ETTBLMEmgR6BFhkNodHRwOi8vY3JsLm1pY3Jv
'' SIG '' c29mdC5jb20vcGtpL2NybC9wcm9kdWN0cy9NaWNyb3Nv
'' SIG '' ZnRUaW1lU3RhbXBQQ0EuY3JsMFgGCCsGAQUFBwEBBEww
'' SIG '' SjBIBggrBgEFBQcwAoY8aHR0cDovL3d3dy5taWNyb3Nv
'' SIG '' ZnQuY29tL3BraS9jZXJ0cy9NaWNyb3NvZnRUaW1lU3Rh
'' SIG '' bXBQQ0EuY3J0MBMGA1UdJQQMMAoGCCsGAQUFBwMIMA0G
'' SIG '' CSqGSIb3DQEBBQUAA4IBAQBi9AUNT+cba4LnUgzfeYyo
'' SIG '' VYEzl9Okysn+r0jbe9pveihPx9C3idjRppnMkVYAOzjo
'' SIG '' tzIv7vnPa9mY3tYC9UJYUmuO2kDcFqCz1L8mFctIQszT
'' SIG '' /bT65ESJZg9CDl73BJ8jSFu0iUHE2mz4NvQ/wh4V35hM
'' SIG '' AMSy7N5fAQFDnLhC1iLIk5qeyaUYZ/xHhB6RXZvydvex
'' SIG '' jnIdgHY8NhFGyn46SPOv40n2FzONuwBjgpxXo1anw26Q
'' SIG '' oz6Vt/xk0V0YwgsBUGOn/PPRgJ6E4zI03VeVQKNtftrr
'' SIG '' LloItBAGXjLJCekygnEWp/rGr2aYifnZnrVfECOjd4Dy
'' SIG '' TdYoAHJQhbzlMIIFvDCCA6SgAwIBAgIKYTMmGgAAAAAA
'' SIG '' MTANBgkqhkiG9w0BAQUFADBfMRMwEQYKCZImiZPyLGQB
'' SIG '' GRYDY29tMRkwFwYKCZImiZPyLGQBGRYJbWljcm9zb2Z0
'' SIG '' MS0wKwYDVQQDEyRNaWNyb3NvZnQgUm9vdCBDZXJ0aWZp
'' SIG '' Y2F0ZSBBdXRob3JpdHkwHhcNMTAwODMxMjIxOTMyWhcN
'' SIG '' MjAwODMxMjIyOTMyWjB5MQswCQYDVQQGEwJVUzETMBEG
'' SIG '' A1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9u
'' SIG '' ZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
'' SIG '' MSMwIQYDVQQDExpNaWNyb3NvZnQgQ29kZSBTaWduaW5n
'' SIG '' IFBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoC
'' SIG '' ggEBALJyWVwZMGS/HZpgICBCmXZTbD4b1m/My/Hqa/6X
'' SIG '' FhDg3zp0gxq3L6Ay7P/ewkJOI9VyANs1VwqJyq4gSfTw
'' SIG '' aKxNS42lvXlLcZtHB9r9Jd+ddYjPqnNEf9eB2/O98jak
'' SIG '' yVxF3K+tPeAoaJcap6Vyc1bxF5Tk/TWUcqDWdl8ed0WD
'' SIG '' hTgW0HNbBbpnUo2lsmkv2hkL/pJ0KeJ2L1TdFDBZ+NKN
'' SIG '' Yv3LyV9GMVC5JxPkQDDPcikQKCLHN049oDI9kM2hOAaF
'' SIG '' XE5WgigqBTK3S9dPY+fSLWLxRT3nrAgA9kahntFbjCZT
'' SIG '' 6HqqSvJGzzc8OJ60d1ylF56NyxGPVjzBrAlfA9MCAwEA
'' SIG '' AaOCAV4wggFaMA8GA1UdEwEB/wQFMAMBAf8wHQYDVR0O
'' SIG '' BBYEFMsR6MrStBZYAck3LjMWFrlMmgofMAsGA1UdDwQE
'' SIG '' AwIBhjASBgkrBgEEAYI3FQEEBQIDAQABMCMGCSsGAQQB
'' SIG '' gjcVAgQWBBT90TFO0yaKleGYYDuoMW+mPLzYLTAZBgkr
'' SIG '' BgEEAYI3FAIEDB4KAFMAdQBiAEMAQTAfBgNVHSMEGDAW
'' SIG '' gBQOrIJgQFYnl+UlE/wq4QpTlVnkpDBQBgNVHR8ESTBH
'' SIG '' MEWgQ6BBhj9odHRwOi8vY3JsLm1pY3Jvc29mdC5jb20v
'' SIG '' cGtpL2NybC9wcm9kdWN0cy9taWNyb3NvZnRyb290Y2Vy
'' SIG '' dC5jcmwwVAYIKwYBBQUHAQEESDBGMEQGCCsGAQUFBzAC
'' SIG '' hjhodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtpL2Nl
'' SIG '' cnRzL01pY3Jvc29mdFJvb3RDZXJ0LmNydDANBgkqhkiG
'' SIG '' 9w0BAQUFAAOCAgEAWTk+fyZGr+tvQLEytWrrDi9uqEn3
'' SIG '' 61917Uw7LddDrQv+y+ktMaMjzHxQmIAhXaw9L0y6oqhW
'' SIG '' nONwu7i0+Hm1SXL3PupBf8rhDBdpy6WcIC36C1DEVs0t
'' SIG '' 40rSvHDnqA2iA6VW4LiKS1fylUKc8fPv7uOGHzQ8uFaa
'' SIG '' 8FMjhSqkghyT4pQHHfLiTviMocroE6WRTsgb0o9ylSpx
'' SIG '' bZsa+BzwU9ZnzCL/XB3Nooy9J7J5Y1ZEolHN+emjWFbd
'' SIG '' mwJFRC9f9Nqu1IIybvyklRPk62nnqaIsvsgrEA5ljpnb
'' SIG '' 9aL6EiYJZTiU8XofSrvR4Vbo0HiWGFzJNRZf3ZMdSY4t
'' SIG '' vq00RBzuEBUaAF3dNVshzpjHCe6FDoxPbQ4TTj18KUic
'' SIG '' ctHzbMrB7HCjV5JXfZSNoBtIA1r3z6NnCnSlNu0tLxfI
'' SIG '' 5nI3EvRvsTxngvlSso0zFmUeDordEN5k9G/ORtTTF+l5
'' SIG '' xAS00/ss3x+KnqwK+xMnQK3k+eGpf0a7B2BHZWBATrBC
'' SIG '' 7E7ts3Z52Ao0CW0cgDEf4g5U3eWh++VHEK1kmP9QFi58
'' SIG '' vwUheuKVQSdpw5OPlcmN2Jshrg1cnPCiroZogwxqLbt2
'' SIG '' awAdlq3yFnv2FoMkuYjPaqhHMS+a3ONxPdcAfmJH0c6I
'' SIG '' ybgY+g5yjcGjPa8CQGr/aZuW4hCoELQ3UAjWwz0wggYH
'' SIG '' MIID76ADAgECAgphFmg0AAAAAAAcMA0GCSqGSIb3DQEB
'' SIG '' BQUAMF8xEzARBgoJkiaJk/IsZAEZFgNjb20xGTAXBgoJ
'' SIG '' kiaJk/IsZAEZFgltaWNyb3NvZnQxLTArBgNVBAMTJE1p
'' SIG '' Y3Jvc29mdCBSb290IENlcnRpZmljYXRlIEF1dGhvcml0
'' SIG '' eTAeFw0wNzA0MDMxMjUzMDlaFw0yMTA0MDMxMzAzMDla
'' SIG '' MHcxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5n
'' SIG '' dG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVN
'' SIG '' aWNyb3NvZnQgQ29ycG9yYXRpb24xITAfBgNVBAMTGE1p
'' SIG '' Y3Jvc29mdCBUaW1lLVN0YW1wIFBDQTCCASIwDQYJKoZI
'' SIG '' hvcNAQEBBQADggEPADCCAQoCggEBAJ+hbLHf20iSKnxr
'' SIG '' LhnhveLjxZlRI1Ctzt0YTiQP7tGn0UytdDAgEesH1VSV
'' SIG '' FUmUG0KSrphcMCbaAGvoe73siQcP9w4EmPCJzB/LMySH
'' SIG '' nfL0Zxws/HvniB3q506jocEjU8qN+kXPCdBer9CwQgSi
'' SIG '' +aZsk2fXKNxGU7CG0OUoRi4nrIZPVVIM5AMs+2qQkDBu
'' SIG '' h/NZMJ36ftaXs+ghl3740hPzCLdTbVK0RZCfSABKR2YR
'' SIG '' JylmqJfk0waBSqL5hKcRRxQJgp+E7VV4/gGaHVAIhQAQ
'' SIG '' MEbtt94jRrvELVSfrx54QTF3zJvfO4OToWECtR0Nsfz3
'' SIG '' m7IBziJLVP/5BcPCIAsCAwEAAaOCAaswggGnMA8GA1Ud
'' SIG '' EwEB/wQFMAMBAf8wHQYDVR0OBBYEFCM0+NlSRnAK7UD7
'' SIG '' dvuzK7DDNbMPMAsGA1UdDwQEAwIBhjAQBgkrBgEEAYI3
'' SIG '' FQEEAwIBADCBmAYDVR0jBIGQMIGNgBQOrIJgQFYnl+Ul
'' SIG '' E/wq4QpTlVnkpKFjpGEwXzETMBEGCgmSJomT8ixkARkW
'' SIG '' A2NvbTEZMBcGCgmSJomT8ixkARkWCW1pY3Jvc29mdDEt
'' SIG '' MCsGA1UEAxMkTWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNh
'' SIG '' dGUgQXV0aG9yaXR5ghB5rRahSqClrUxzWPQHEy5lMFAG
'' SIG '' A1UdHwRJMEcwRaBDoEGGP2h0dHA6Ly9jcmwubWljcm9z
'' SIG '' b2Z0LmNvbS9wa2kvY3JsL3Byb2R1Y3RzL21pY3Jvc29m
'' SIG '' dHJvb3RjZXJ0LmNybDBUBggrBgEFBQcBAQRIMEYwRAYI
'' SIG '' KwYBBQUHMAKGOGh0dHA6Ly93d3cubWljcm9zb2Z0LmNv
'' SIG '' bS9wa2kvY2VydHMvTWljcm9zb2Z0Um9vdENlcnQuY3J0
'' SIG '' MBMGA1UdJQQMMAoGCCsGAQUFBwMIMA0GCSqGSIb3DQEB
'' SIG '' BQUAA4ICAQAQl4rDXANENt3ptK132855UU0BsS50cVtt
'' SIG '' DBOrzr57j7gu1BKijG1iuFcCy04gE1CZ3XpA4le7r1ia
'' SIG '' HOEdAYasu3jyi9DsOwHu4r6PCgXIjUji8FMV3U+rkuTn
'' SIG '' jWrVgMHmlPIGL4UD6ZEqJCJw+/b85HiZLg33B+JwvBhO
'' SIG '' nY5rCnKVuKE5nGctxVEO6mJcPxaYiyA/4gcaMvnMMUp2
'' SIG '' MT0rcgvI6nA9/4UKE9/CCmGO8Ne4F+tOi3/FNSteo7/r
'' SIG '' vH0LQnvUU3Ih7jDKu3hlXFsBFwoUDtLaFJj1PLlmWLMt
'' SIG '' L+f5hYbMUVbonXCUbKw5TNT2eb+qGHpiKe+imyk0Bnca
'' SIG '' Ysk9Hm0fgvALxyy7z0Oz5fnsfbXjpKh0NbhOxXEjEiZ2
'' SIG '' CzxSjHFaRkMUvLOzsE1nyJ9C/4B5IYCeFTBm6EISXhrI
'' SIG '' niIh0EPpK+m79EjMLNTYMoBMJipIJF9a6lbvpt6Znco6
'' SIG '' b72BJ3QGEe52Ib+bgsEnVLaxaj2JoXZhtG6hE6a/qkfw
'' SIG '' Em/9ijJssv7fUciMI8lmvZ0dhxJkAj0tr1mPuOQh5bWw
'' SIG '' ymO0eFQF1EEuUKyUsKV4q7OglnUa2ZKHE3UiLzKoCG6g
'' SIG '' W4wlv6DvhMoh1useT8ma7kng9wFlb4kLfchpyOZu6qeX
'' SIG '' zjEp/w7FW1zYTRuh2Povnj8uVRZryROj/TGCBHcwggRz
'' SIG '' AgEBMIGHMHkxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpX
'' SIG '' YXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYD
'' SIG '' VQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xIzAhBgNV
'' SIG '' BAMTGk1pY3Jvc29mdCBDb2RlIFNpZ25pbmcgUENBAgph
'' SIG '' GcyTAAEAAABmMAkGBSsOAwIaBQCggaQwGQYJKoZIhvcN
'' SIG '' AQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEO
'' SIG '' MAwGCisGAQQBgjcCARUwIwYJKoZIhvcNAQkEMRYEFIaI
'' SIG '' dxnyg8YownJSpJ4e+JiZWAjoMEQGCisGAQQBgjcCAQwx
'' SIG '' NjA0oBaAFABVAEQASQAgAFcAaQB6AGEAcgBkoRqAGGh0
'' SIG '' dHA6Ly93d3cubWljcm9zb2Z0LmNvbTANBgkqhkiG9w0B
'' SIG '' AQEFAASCAQBqs+BuZ5DrAwaTjWp3FO28gzcah4KtJq5f
'' SIG '' ieZ1n/uE79Ng+7Vwpml7rRW4Jzvh/vJkP46a9sMuaQl6
'' SIG '' lEFZObETRAXsPecsM5P3aLKb4FpJfbNL/Dv+xGc1UbjI
'' SIG '' xhpbM2Zqq3D8cjou+rulx325HKgqVFeyAUwJiiG6MLK6
'' SIG '' qeBL49CCX+5tddKylKKSyljKGD7SvLpyivNq+WIDfRBc
'' SIG '' iKwBU5ZRwpu8snRJGhgKlwV/EmkUxW9wzm3OJ2dy7HBJ
'' SIG '' wkExrUztmpgQmuX0yebiCA43hBbrxOxgm/63mPt1824O
'' SIG '' hUL2kysbGk+zHlob8pBL0fTjPTXaoV1fXX39uMkDb560
'' SIG '' oYICHTCCAhkGCSqGSIb3DQEJBjGCAgowggIGAgEBMIGF
'' SIG '' MHcxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5n
'' SIG '' dG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVN
'' SIG '' aWNyb3NvZnQgQ29ycG9yYXRpb24xITAfBgNVBAMTGE1p
'' SIG '' Y3Jvc29mdCBUaW1lLVN0YW1wIFBDQQIKYQUTNgAAAAAA
'' SIG '' GjAHBgUrDgMCGqBdMBgGCSqGSIb3DQEJAzELBgkqhkiG
'' SIG '' 9w0BBwEwHAYJKoZIhvcNAQkFMQ8XDTEyMDQxMDIzMjMy
'' SIG '' NFowIwYJKoZIhvcNAQkEMRYEFE79u+TtumwAEEB2ox4U
'' SIG '' mZFV9D0cMA0GCSqGSIb3DQEBBQUABIIBABIHrX/I7qMg
'' SIG '' UkoptAJzt2kCKJSsYFUo4yJl6YXAHqOtBSj5e3etZZIE
'' SIG '' QViqhX19e5EVMS0YVwGLGemyC5eW3mr39aB3B52KrrrR
'' SIG '' mZZkvZ2dsq+fDhnhsELqkuU9MAQD1H72KrVLpaTy1iIr
'' SIG '' AxLA0XrOJ4HK8bQb51PU7Q4U1OIqo3yuGxjlprGnLvzc
'' SIG '' C6NcGwfnAxLz/dKXJh3hwzyThGVRIGIvFwivrJdS30Vr
'' SIG '' hL1GkKnRepB9OQ1cWsnAOk0/Ah4a+PaKL5KiLKo2/uqn
'' SIG '' +pQgv+B+zpsME5NYoUQsWYK1bKU58f3zNRPXhgHn3sCB
'' SIG '' l91m5J/f0RQKYZtf/d8Op0A=
'' SIG '' End signature block
